@@ -59,6 +59,7 @@
 #include "util.h"
 #include "startup.h"
 #include "slrndir.h"
+#include "vfile.h"
 
 /*}}}*/
 
@@ -85,6 +86,7 @@ int Slrn_No_Backups = 0;
 int Slrn_Prompt_Next_Group = 1;
 
 int Slrn_Unsubscribe_New_Groups = 0;
+int Slrn_Check_New_Groups = 1;
 
 SLKeyMap_List_Type *Slrn_Group_Keymap;
 int *Slrn_Prefix_Arg_Ptr;
@@ -100,7 +102,7 @@ static int Groups_Hidden;	       /* if true, hide groups with no arts */
 
 typedef struct Unsubscribed_Slrn_Group_Type
 {
-   char name[MAX_GROUP_NAME_LEN + 1];
+   char *group_name;
    struct Unsubscribed_Slrn_Group_Type *next;
 }
 Unsubscribed_Slrn_Group_Type;
@@ -254,7 +256,7 @@ void slrn_mark_article_as_read (char *group, long num) /*{{{*/
    
    while (g != NULL)
      {
-	if ((g->hash == hash) && !strcmp (group, g->name))
+	if ((g->hash == hash) && !strcmp (group, g->group_name))
 	  {
 	     /* If it looks like we have read this group, mark it read. */
 	     if (((g->flags & GROUP_UNSUBSCRIBED) == 0)
@@ -278,7 +280,7 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
    
    if (g == NULL) return -1;
    
-   group = g->name;
+   group = g->group_name;
    
    slrn_message_now ("Selecting %s ...", group);
    
@@ -384,6 +386,24 @@ static void free_group_ranges (Slrn_Group_Type *g) /*{{{*/
 
 /*}}}*/
 
+static void free_newsgroup_type (Slrn_Group_Type *g)
+{
+   if (g == NULL)
+     return;
+   slrn_free (g->descript);
+   free_group_ranges (g);
+   slrn_free (g->group_name);
+   slrn_free ((char *) g);
+}
+
+static void free_unsubscribed_group_type (Unsubscribed_Slrn_Group_Type *ug)
+{
+   if (ug == NULL)
+     return;
+   slrn_free (ug->group_name);
+   slrn_free ((char *) ug);
+}
+
 void slrn_catchup_group (void) /*{{{*/
 {
    if (Slrn_Group_Current_Group == NULL) return;
@@ -423,9 +443,9 @@ static Slrn_Group_Type *find_group_entry (char *name, unsigned int len) /*{{{*/
    
    while (g != NULL)
      {
-	if ((g->hash == hash) && !strncmp (name, g->name, len))
+	if ((g->hash == hash) && !strncmp (name, g->group_name, len))
 	  {
-	     if (len == strlen (g->name)) break;;
+	     if (len == strlen (g->group_name)) break;
 	  }
 	g = g->hash_next;
      }
@@ -448,24 +468,23 @@ static Slrn_Group_Type *create_group_entry (char *name, unsigned int len, /*{{{*
      }
    
    g = (Slrn_Group_Type *) slrn_safe_malloc (sizeof (Slrn_Group_Type));
-
-   if (len > MAX_GROUP_NAME_LEN) len = MAX_GROUP_NAME_LEN;
-   strncpy (g->name, name, len);
-   g->name [len] = 0;
+   g->group_name = slrn_safe_malloc (len + 1);
+   strncpy (g->group_name, name, len);
+   g->group_name [len] = 0;
    
    if (query_server)
      {
 	int status;
 	
-	status = Slrn_Server_Obj->sv_select_group (g->name, &min, &max);
+	status = Slrn_Server_Obj->sv_select_group (g->group_name, &min, &max);
 	if (status != OK_GROUP)
 	  {
 	     if (status != -1)
 	       {
 		  Slrn_Groups_Dirty = 1;
-		  slrn_error ("Group %s may be bogus.", g->name);
+		  slrn_error ("Group %s may be bogus.", g->group_name);
 	       }
-	     SLFREE (g);
+	     free_newsgroup_type (g);
 	     return NULL;
 	  }
      }
@@ -513,11 +532,13 @@ static int add_group (char *name, unsigned int len, /*{{{*/
      {
 	if (Slrn_List_Active_File)
 	  {
-	     char namebuf[MAX_GROUP_NAME_LEN + 1];
-	     if (len > MAX_GROUP_NAME_LEN) len = MAX_GROUP_NAME_LEN;
-	     strncpy (namebuf, name, len);
-	     namebuf[len] = 0;
-	     fprintf (stderr, "Group %s is bogus,  Ignoring it.\r\n", namebuf);
+	     char *tmp_name;
+
+	     if (NULL != (tmp_name = slrn_strnmalloc (name, len, 1)))
+	       {
+		  fprintf (stderr, "Group %s is bogus,  Ignoring it.\r\n", tmp_name);
+		  slrn_free (tmp_name);
+	       }
 	     return -1;
 	  }
 	else g = create_group_entry (name, len, -1, -1,
@@ -613,9 +634,7 @@ static void free_all_groups (void) /*{{{*/
    while (g != NULL)
      {
 	nextg = g->next;
-	slrn_free (g->descript);
-	free_group_ranges (g);
-	SLFREE(g);
+	free_newsgroup_type (g);
 	g = nextg;
      }
    Groups = NULL;
@@ -716,10 +735,8 @@ static void add_unsubscribed_group (char *name) /*{{{*/
    while (*p > ' ') p++;
    *p = 0;
    len = p - name;
-   
-   if (len > MAX_GROUP_NAME_LEN) len = MAX_GROUP_NAME_LEN;
-   strncpy (g->name, name, len);
-   g->name[len] = 0;
+
+   g->group_name = slrn_safe_strmalloc (name);
 }
 
 /*}}}*/
@@ -746,8 +763,8 @@ int slrn_group_search (char *str) /*{{{*/
 	
 	if ((g->flags & GROUP_HIDDEN) == 0)
 	  {
-	     if ((NULL != SLsearch ((unsigned char *) g->name,
-				   (unsigned char *) g->name + strlen (g->name),
+	     if ((NULL != SLsearch ((unsigned char *) g->group_name,
+				    (unsigned char *) g->group_name + strlen (g->group_name),
 				   &st))
 		 || ((NULL != g->descript)
 		     && (NULL != SLsearch ((unsigned char *) g->descript,
@@ -822,7 +839,7 @@ int slrn_group_select_group (void) /*{{{*/
 	&& (Slrn_Query_Group_Cutoff > 0)))
      {
 	char int_prompt_buf[256];
-	sprintf (int_prompt_buf, "%s: Read how many?", Slrn_Group_Current_Group->name);
+	sprintf (int_prompt_buf, "%s: Read how many?", Slrn_Group_Current_Group->group_name);
 	if ((-1 == slrn_read_integer (int_prompt_buf, &n, &n))
 	    || (n <= 0))
 	  {
@@ -954,11 +971,11 @@ static void group_up (void) /*{{{*/
 
 static void refresh_groups_cmd (void) /*{{{*/
 {
-   char name[MAX_GROUP_NAME_LEN + 1];
+   char *group_name;
    
-   *name = 0;
-   if (Slrn_Group_Current_Group != NULL) 
-     strcpy (name, Slrn_Group_Current_Group->name);
+   group_name = NULL;
+   if (Slrn_Group_Current_Group != NULL)
+     group_name = slrn_safe_strmalloc (Slrn_Group_Current_Group->group_name);
    
    /* slrn_set_suspension (1); */
    save_newsrc_cmd ();
@@ -973,8 +990,11 @@ static void refresh_groups_cmd (void) /*{{{*/
      {
 	slrn_quit (0);
      }
-   if (*name)
-     (void) find_group (name);
+   if (group_name != NULL)
+     {
+	(void) find_group (group_name);
+	slrn_free (group_name);
+     }
    /* slrn_set_suspension (0); */
 
    init_group_win_struct ();
@@ -1086,7 +1106,7 @@ static void move_group_cmd (void) /*{{{*/
    
    while (1)
      {
-	slrn_message_now ("Moving %s. Press RETURN when finished.", Slrn_Group_Current_Group->name);
+	slrn_message_now ("Moving %s. Press RETURN when finished.", Slrn_Group_Current_Group->group_name);
 	
 	key = SLang_do_key (Slrn_Group_Keymap, (int (*)(void)) SLang_getkey);
 	
@@ -1155,7 +1175,7 @@ static void subscribe (void) /*{{{*/
      {
 	if (g->flags & GROUP_UNSUBSCRIBED)
 	  {
-	     if (NULL != slrn_regexp_match (re, g->name))
+	     if (NULL != slrn_regexp_match (re, g->group_name))
 	       {
 		  g->flags &= ~GROUP_HIDDEN;
 		  g->flags &= ~GROUP_UNSUBSCRIBED;
@@ -1175,7 +1195,7 @@ static void catch_up (void) /*{{{*/
    if ((Slrn_Group_Current_Group == NULL)
        || (Slrn_User_Wants_Confirmation
 	   && (Slrn_Batch == 0)
-	   && slrn_get_yesno(1, "Mark %s as read", Slrn_Group_Current_Group->name) <= 0))
+	   && slrn_get_yesno(1, "Mark %s as read", Slrn_Group_Current_Group->group_name) <= 0))
      return;
    
    slrn_catchup_group ();
@@ -1190,7 +1210,7 @@ static void uncatch_up (void) /*{{{*/
    if ((Slrn_Group_Current_Group == NULL)
        || (Slrn_User_Wants_Confirmation 
 	   && (Slrn_Batch == 0)
-	   && slrn_get_yesno(1, "Mark %s as un-read", Slrn_Group_Current_Group->name) <= 0))
+	   && slrn_get_yesno(1, "Mark %s as un-read", Slrn_Group_Current_Group->group_name) <= 0))
      return;
  
    slrn_uncatchup_group ();   
@@ -1225,7 +1245,7 @@ static void unsubscribe (void) /*{{{*/
      {
 	if ((g->flags & GROUP_UNSUBSCRIBED) == 0)
 	  {
-	     if (NULL != (slrn_regexp_match (re, g->name)))
+	     if (NULL != (slrn_regexp_match (re, g->group_name)))
 	       {
 		  g->flags &= ~GROUP_HIDDEN;
 		  g->flags |= (GROUP_TOUCHED | GROUP_UNSUBSCRIBED);
@@ -1293,7 +1313,7 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 	  {
 	     if (g->flags & GROUP_UNSUBSCRIBED)
 	       {
-		  if (NULL != slrn_regexp_match (re, g->name))
+		  if (NULL != slrn_regexp_match (re, g->group_name))
 		    {
 		       if (first_found == NULL) first_found = g;
 		       g->flags &= ~GROUP_HIDDEN;
@@ -1433,7 +1453,12 @@ void slrn_post_cmd (void) /*{{{*/
        (slrn_get_yesno (1, "Are you sure that you want to post") <= 0))
      return;
    
-   if (Slrn_Group_Current_Group == NULL) name = ""; else name = Slrn_Group_Current_Group->name;
+   if (Slrn_Group_Current_Group == NULL)
+     name = "";
+   else name = Slrn_Group_Current_Group->group_name;
+
+   if (strlen (name) >= sizeof (group))
+     name = "";
    strcpy (group, name);
    if (slrn_read_input ("Newsgroup", NULL, group, 1, 1) <= 0) return;
    *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1, 0) <= 0) return;
@@ -1784,7 +1809,7 @@ static void add_group_description (unsigned char *s, unsigned char *smax, /*{{{*
    g = Group_Hash_Table[hash % GROUP_HASH_TABLE_SIZE];
    while (g != NULL)
      {
-	if ((g->hash == hash) && (!strncmp (g->name, (char *) s, (unsigned int) (smax - s))))
+	if ((g->hash == hash) && (!strncmp (g->group_name, (char *) s, (unsigned int) (smax - s))))
 	  {
 	     /* Sometimes these get repeated --- not by slrn but on the server! */
 	     slrn_free (g->descript);
@@ -1952,7 +1977,7 @@ int slrn_check_new_groups (int create_flag) /*{{{*/
    FILE *fp;
    time_t tloc;
    struct tm *tm_struct;
-   char line[SLRN_MAX_PATH_LEN];
+   char line[NNTP_BUFFER_SIZE];
    char file[SLRN_MAX_PATH_LEN];
    int num;
    char *p;
@@ -1970,7 +1995,7 @@ int slrn_check_new_groups (int create_flag) /*{{{*/
    
    
    if ((create_flag == 0)
-       && (NULL != (fp = slrn_open_home_file (file, "r", line, 0))))
+       && (NULL != (fp = slrn_open_home_file (file, "r", NULL, 0))))
      {
 	char ch;
 	int i;
@@ -2078,9 +2103,9 @@ I will patch the file up for you.\n", file);
      }
    
    
-   if (NULL == (fp = slrn_open_home_file (file, "w", line, 1)))
+   if (NULL == (fp = slrn_open_home_file (file, "w", NULL, 1)))
      {
-	slrn_exit_error ("Unable to open %s to record date.", line);
+	slrn_exit_error ("Unable to open %s to record date.", file);
      }
 
    /* According to rfc977, the year must be in the form YY with the closest
@@ -2169,20 +2194,86 @@ static int parse_active_line (char *name, unsigned int *lenp, /*{{{*/
 
 /*}}}*/
 
+static int read_and_parse_newsrc_file (void)
+{
+   Slrn_Group_Type *last_group = NULL;
+   VFILE *vp;
+   char *vline;
+   unsigned int vlen;
+   char file[SLRN_MAX_PATH_LEN];
+
+   if (Unsubscribed_Groups != NULL)
+     {
+	unsigned int subscribe_flag;
+	Unsubscribed_Slrn_Group_Type *ug = Unsubscribed_Groups, *ugnext;
+	     
+	if (Slrn_Unsubscribe_New_Groups)
+	  subscribe_flag = GROUP_UNSUBSCRIBED | GROUP_NEW_GROUP_FLAG;
+	else subscribe_flag = GROUP_NEW_GROUP_FLAG;
+	     
+	while (ug != NULL)
+	  {
+	     ugnext = ug->next;
+	     
+	     if ((-1 != add_group (ug->group_name, strlen (ug->group_name), subscribe_flag, 0))
+		 && Slrn_List_Active_File)
+	       last_group = place_group_in_newsrc_order (last_group);
+	     
+	     free_unsubscribed_group_type (ug);
+	     ug = ugnext;
+	  }
+	Unsubscribed_Groups = NULL;
+     }
+
+   if ((NULL == (vp = slrn_open_home_vfile (Slrn_Newsrc_File, file)))
+       && (NULL == (vp = slrn_open_home_vfile (".newsrc", file))))
+     slrn_exit_error ("Unable to open %s.", file);
+
+   while (NULL != (vline = vgets (vp, &vlen)))
+     {
+	char *p = vline;
+	char *pmax = p + vlen;
+	char ch;
+
+	while ((p < pmax)
+	       && ((ch = *p) != '!') && (ch != ':'))
+	  p++;
+
+	if ((p == pmax) || (p == vline))
+	  continue;
+     
+	vline[vlen-1] = 0;	       /* kill \n and NULL terminate */
+
+	if (-1 == add_group (vline, (unsigned int) (p - vline),
+			     ((ch == '!') ? GROUP_UNSUBSCRIBED : 0), 0))
+	  continue;
+	     
+	/* perform a re-arrangement to match arrangement in the
+	 * newsrc file
+	 */
+	if (Slrn_List_Active_File && (ch !=  '!'))
+	  last_group = place_group_in_newsrc_order (last_group);
+     }
+   vclose (vp);
+   return 0;
+}
+
+   
 int slrn_read_newsrc (int create_flag) /*{{{*/
 {
-   char file[SLRN_MAX_PATH_LEN];
-   FILE *fp = NULL;
    char line[NNTP_BUFFER_SIZE];
-   register char *p, ch;
    
    if (create_flag)
      {
+	FILE *fp;
+	char file[SLRN_MAX_PATH_LEN];
+
+	/* See if we can open the file. */
 	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "w", file, 1)))
 	  {
 	     slrn_exit_error ("Unable to create %s.", file);
 	  }
-	
+	fclose (fp);
 	fputs ("\n--The next step may take a while if the NNTP connection is slow.--\n\n", stdout);
 	fprintf (stdout, "Creating %s.", file);
 	fflush (stdout);
@@ -2236,7 +2327,8 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 		       add_group (line, len, 0, 1);
 		    }
 		  else if ((*line == 'a')
-			   && (!strncmp (line, "alt.test", 8)))
+			   && (!strncmp (line, "alt.test", 8))
+			   && (line[8] <= ' '))
 		    {
 		       add_group (line, len, 0, 1);
 		    }
@@ -2245,64 +2337,9 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 	  }
      }
    
-   if (create_flag == 0)
-     {
-	Slrn_Group_Type *last_group = NULL;
-	
-	if (Unsubscribed_Groups != NULL)
-	  {
-	     unsigned int subscribe_flag;
-	     Unsubscribed_Slrn_Group_Type *ug = Unsubscribed_Groups, *ugnext;
-	     
-	     if (Slrn_Unsubscribe_New_Groups)
-	       subscribe_flag = GROUP_UNSUBSCRIBED | GROUP_NEW_GROUP_FLAG;
-	     else subscribe_flag = GROUP_NEW_GROUP_FLAG;
-	     
-	     while (ug != NULL)
-	       {
-		  ugnext = ug->next;
-		  
-		  if ((-1 != add_group (ug->name, strlen (ug->name), subscribe_flag, 0))
-		      && Slrn_List_Active_File)
-		    last_group = place_group_in_newsrc_order (last_group);
-		  
-		  SLFREE (ug);
-		  ug = ugnext;
-	       }
-	     Unsubscribed_Groups = NULL;
-	  }
-	
-	if ((NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "r", file, 0)))
-	    && (NULL == (fp = slrn_open_home_file (".newsrc", "r", file, 0))))
-	  {
-	     sprintf (line, "Unable to open %s.", file);
-	     slrn_exit_error (line);
-	  }
-	
-	while (fgets (line, sizeof(line) - 1, fp) != NULL)
-	  {
-	     p = line;
-	     while (((ch = *p) != '!') && (ch != ':') && (ch != 0))
-	       {
-		  p++;
-	       }
-	     if ((ch == 0) || (p == line)) continue;
-	     if (-1 == add_group (line, (unsigned int) (p - line),
-				  ((ch == '!') ? GROUP_UNSUBSCRIBED : 0),
-				  0))
-	       continue;
-	     
-	     /* perform a re-arrangement to match arrangement in the
-	      * newsrc file
-	      */
-	     if (Slrn_List_Active_File && (ch !=  '!'))
-	       {
-		  last_group = place_group_in_newsrc_order (last_group);
-	       }
-	  }
-     }
-   
-   slrn_fclose (fp);
+   if ((create_flag == 0)
+       && (-1 == read_and_parse_newsrc_file ()))
+     slrn_exit_error ("Unable to read newsrc file");
    
    Slrn_Group_Current_Group = Groups;
 
@@ -2450,7 +2487,7 @@ int slrn_write_newsrc (void) /*{{{*/
 		       g = g->next;
 		       continue;
 		    }
-		  if ((EOF == fputs (g->name, fp))
+		  if ((EOF == fputs (g->group_name, fp))
 		      || (EOF == putc (':', fp)))
 		    goto write_error;
 	       }
@@ -2472,7 +2509,7 @@ int slrn_write_newsrc (void) /*{{{*/
 			    continue;
 			 }
 		    }
-		  if ((EOF == fputs (g->name, fp))
+		  if ((EOF == fputs (g->group_name, fp))
 		      || (EOF == putc ('!', fp)))
 		    goto write_error;
 	       }
@@ -2606,7 +2643,7 @@ static void group_update_screen (void) /*{{{*/
 				 : ((g->flags & GROUP_NEW_GROUP_FLAG) ? 'N' : ' ')),
 				g->unread);
 		  slrn_set_color (GROUP_COLOR);
-		  SLsmg_printf ("%s", g->name);
+		  SLsmg_printf ("%s", g->group_name);
 		  slrn_set_color (0);
 		  SLsmg_erase_eol ();
 		  if (Slrn_Group_Display_Descriptions)
