@@ -1,8 +1,23 @@
 /* -*- mode: C; mode: fold -*- */
-/*  Copyright (c) 1996 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -14,7 +29,9 @@
 
 #include <slang.h>
 #include <stdarg.h>
-
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
 #include <ctype.h>
 
 #include "jdmacros.h"
@@ -29,48 +46,68 @@
 #include "util.h"
 #include "server.h"
 
-/*{{{ Low level grouplens tcp stuff */
-
 
 
 #define GL_MAX_RESPONSE_LINE_LEN	1024
 
 /*{{{ GL_Type structure */
 
-typedef struct
+typedef struct GL_Type /*{{{*/
 {
-#define GL_MAX_HOSTNAME_LEN		255
-   char hostname[GL_MAX_HOSTNAME_LEN + 1];
+   char *hostname;
    int port;
    
    char *token;
    char *pseudoname;
    int logged_in;
    SLTCP_Type *tcp;
+   struct GL_Type *next;
 }
-GL_Type;
-
-static GL_Type GL_Server;
 
 /*}}}*/
 
-typedef struct
+GL_Type;
+
+static GL_Type GL_Nameserver;
+static GL_Type *GL_Server_List;
+/*}}}*/
+
+typedef struct /*{{{*/
 {
    char *msgid;
    float pred;
    float confhigh;
    float conflow;
 }
+
+/*}}}*/
 GL_Prediction_Type;
 
-typedef struct
+typedef struct /*{{{*/
 {
    char *msgid;
    int rating;
    int saw_article;
 }
+
+/*}}}*/
 GL_Rating_Type;
 
+char *Slrn_GroupLens_Host;
+int Slrn_GroupLens_Port;
+char *Slrn_GroupLens_Pseudoname;
+
+typedef struct GL_Newsgroup_List_Type /*{{{*/
+{
+   char *group;
+   GL_Type *gl;
+   struct GL_Newsgroup_List_Type *next;
+}
+
+/*}}}*/
+GL_Newsgroup_List_Type;
+
+static GL_Newsgroup_List_Type *Newsgroup_List = NULL;
 
 /*{{{ Error codes and Error functions */
 
@@ -87,19 +124,22 @@ static int GL_Error;
 #define GL_ERROR_TOKEN		       10
 #define GL_ERROR_NEWSGROUP	       11
 
-typedef struct
+typedef struct /*{{{*/
 {
    char *err;
    unsigned int len;
    int errcode;
 }
+
+/*}}}*/
+
 GL_Error_Type;
 
 
 
 /*{{{ Utility functions */
 
-static char *make_nstring (char *s, unsigned int len)
+static char *make_nstring (char *s, unsigned int len) /*{{{*/
 {
    char *s1;
    
@@ -115,13 +155,17 @@ static char *make_nstring (char *s, unsigned int len)
    return s1;
 }
 
-static char *make_string (char *s)
+/*}}}*/
+
+static char *make_string (char *s) /*{{{*/
 {
    if (s == NULL) return s;
    return make_nstring (s, strlen (s));
 }
 
-static char *skip_whitespace (char *s)
+/*}}}*/
+
+static char *skip_whitespace (char *s) /*{{{*/
 {
    char ch;
    
@@ -129,7 +173,9 @@ static char *skip_whitespace (char *s)
    return s;
 }
 
-static char *skip_nonwhitespace (char *s)
+/*}}}*/
+
+static char *skip_nonwhitespace (char *s) /*{{{*/
 {
    char ch;
    
@@ -139,7 +185,9 @@ static char *skip_nonwhitespace (char *s)
 
 /*}}}*/
 
-static char *gl_get_error (void)
+/*}}}*/
+
+static char *gl_get_error (void) /*{{{*/
 {
    switch (GL_Error)
      {
@@ -158,13 +206,19 @@ static char *gl_get_error (void)
    return "Unknown Error";
 }
 
+/*}}}*/
 
-static void close_gl (GL_Type *gl)
+
+static void close_gl (GL_Type *gl) /*{{{*/
 {
-   if (gl != NULL) sltcp_close (gl->tcp);
+   if (gl == NULL) return;
+   sltcp_close (gl->tcp);
+   gl->tcp = NULL;
 }
 
-static int get_gl_response (GL_Type *gl, char *buf, unsigned int len)
+/*}}}*/
+
+static int get_gl_response (GL_Type *gl, char *buf, unsigned int len) /*{{{*/
 {
    char *b;
    
@@ -191,14 +245,19 @@ static int get_gl_response (GL_Type *gl, char *buf, unsigned int len)
    return 0;
 }
 
-static void handle_error_response (GL_Type *gl, GL_Error_Type *tbl)
+/*}}}*/
+
+static void handle_error_response (GL_Type *gl, GL_Error_Type *tbl) /*{{{*/
 {
    char *s, *s1;
    char buf [GL_MAX_RESPONSE_LINE_LEN];
    
    if (-1 == get_gl_response (gl, buf, sizeof (buf)))
      return;
-	
+   
+   if (tbl == NULL)
+     return;
+
    s = skip_whitespace (buf);
    
    GL_Error = GL_ERROR_UNKNOWN;
@@ -235,9 +294,11 @@ static void handle_error_response (GL_Type *gl, GL_Error_Type *tbl)
    
    close_gl (gl);
 }
+
+/*}}}*/
+
    
-   
-static int is_error (char *buf)
+static int is_error (char *buf) /*{{{*/
 {
    if (!slrn_case_strncmp ((unsigned char *) buf, (unsigned char *) "ERROR", 5))
      return 1;
@@ -247,11 +308,12 @@ static int is_error (char *buf)
 
 /*}}}*/
 
+/*}}}*/
 
 /*{{{ parse_keyword_eqs_value */
 
 /* Scan buf looking for keyword=VALUE */
-static int parse_keyword_eqs_value (char *buf, char *kw, char **value, unsigned int *len)
+static int parse_keyword_eqs_value (char *buf, char *kw, char **value, unsigned int *len) /*{{{*/
 {
    char *b, *b1, *v, *v1;
    char ch;
@@ -300,13 +362,13 @@ static int parse_keyword_eqs_value (char *buf, char *kw, char **value, unsigned
      }
 }
 
+/*}}}*/
+
 /*}}}*/
 /*{{{ low level GL server functions */
 
-
-
    
-static int send_gl_line (GL_Type *gl, char *msg, int flush)
+static int send_gl_line (GL_Type *gl, char *msg, int flush) /*{{{*/
 {
    if (msg == NULL) return 0;
    
@@ -322,19 +384,30 @@ static int send_gl_line (GL_Type *gl, char *msg, int flush)
    return 0;
 }
 
-static int connect_to_gl_host (GL_Type *gl, char *cmd)
+/*}}}*/
+
+static int connect_to_gl_host (GL_Type *gl, char *cmd) /*{{{*/
 {
    char buf [GL_MAX_RESPONSE_LINE_LEN];
-   
+   char *host;
+   int port;
+
    GL_Error = 0;
    
-   if (gl->hostname == NULL)
+   port = gl->port;
+   if (NULL == (host = gl->hostname))
+     {
+	host = Slrn_GroupLens_Host;
+	port = Slrn_GroupLens_Port;
+     }
+   
+   if ((host == NULL) || (port <= 0))
      {
 	GL_Error = GL_ERROR_UNKNOWN;
 	return -1;
      }
    
-   if (NULL == (gl->tcp = sltcp_open_connection (gl->hostname, gl->port)))
+   if (NULL == (gl->tcp = sltcp_open_connection (host, port)))
      return -1;
    
    /* We should be able to read OK ...  If not, we failed to make proper 
@@ -354,8 +427,10 @@ static int connect_to_gl_host (GL_Type *gl, char *cmd)
    return send_gl_line (gl, cmd, 1);
 }
 
+/*}}}*/
 
-static int start_command (GL_Type *gl, char *fmt, ...)
+
+static int start_command (GL_Type *gl, char *fmt, ...) /*{{{*/
 {
    va_list ap;
    int ret;
@@ -377,9 +452,11 @@ static int start_command (GL_Type *gl, char *fmt, ...)
    return 0;
 }
 
+/*}}}*/
 
 
-static int check_for_error_response (GL_Type *gl, GL_Error_Type *tbl)
+
+static int check_for_error_response (GL_Type *gl, GL_Error_Type *tbl) /*{{{*/
 {
    char response [GL_MAX_RESPONSE_LINE_LEN];
    
@@ -394,10 +471,12 @@ static int check_for_error_response (GL_Type *gl, GL_Error_Type *tbl)
    return 0;
 }
 
+/*}}}*/
+
 /* Open connection, send command check for error and handle it via table
  * if error occurs.
  */
-static int gl_send_command (GL_Type *gl, GL_Error_Type *tbl, char *fmt, ...)
+static int gl_send_command (GL_Type *gl, GL_Error_Type *tbl, char *fmt, ...) /*{{{*/
 {
    va_list ap;
    int ret;
@@ -417,18 +496,18 @@ static int gl_send_command (GL_Type *gl, GL_Error_Type *tbl, char *fmt, ...)
 	return -1;
      }
    
-   if (tbl == NULL) return 0;
-
    return check_for_error_response (gl, tbl);
 }
-   
+
+/*}}}*/
+
 
 
 /*}}}*/
 
 /*{{{ login functions */
 
-static GL_Error_Type Login_Error_Table [] = 
+static GL_Error_Type Login_Error_Table [] = /*{{{*/
 {
    {":busy",		5,	GL_ELOGIN_BUSY},
    {":unavailable",	12,	GL_ELOGIN_UNAVAILABLE},
@@ -437,9 +516,11 @@ static GL_Error_Type Login_Error_Table [] =
    {NULL, 0, 0}
 };
 
+/*}}}*/
 
 
-static int login (GL_Type *gl)
+
+static int login (GL_Type *gl) /*{{{*/
 {
    char response [GL_MAX_RESPONSE_LINE_LEN];
    char *value; 
@@ -447,7 +528,9 @@ static int login (GL_Type *gl)
 
    gl->logged_in = 0;
    
-   if (-1 == gl_send_command (gl, Login_Error_Table, "LOGIN %s\r\n", gl->pseudoname))
+   
+   if (-1 == gl_send_command (gl, Login_Error_Table, "LOGIN %s :client=\"slrn %s\"\r\n",
+			      gl->pseudoname, Slrn_Version))
      return -1;
    
    /* parse response to get token, etc... */
@@ -479,7 +562,9 @@ static int login (GL_Type *gl)
    return 0;
 }
 
-static int validate_token (GL_Type *gl)
+/*}}}*/
+
+static int validate_token (GL_Type *gl) /*{{{*/
 {   
    if ((gl->token == NULL) || (gl->logged_in == 0))
      return login (gl);
@@ -487,10 +572,12 @@ static int validate_token (GL_Type *gl)
    return 0;
 }
 
+/*}}}*/
+
 /*}}}*/
 /*{{{ logout functions */
 
-static void logout (GL_Type *gl)
+static void logout (GL_Type *gl) /*{{{*/
 {
    if (gl->logged_in == 0) return;
    
@@ -499,53 +586,24 @@ static void logout (GL_Type *gl)
 }
 
 /*}}}*/
-/*{{{ gl_open_server */
-
-static int gl_open_server (char *host, int port, char *pname)
-{
-   GL_Type *gl;
-   
-   if (host == NULL) return -1;
-   if (port < 0) return -1;
-   if (pname == NULL) return -1;
-   
-   gl = &GL_Server;
-   memset ((char *) gl, 0, sizeof (GL_Type));
-   
-   if (NULL == (gl->pseudoname = make_string (pname)))
-     return -1;
-   
-   strncpy (gl->hostname, host, GL_MAX_HOSTNAME_LEN);
-   gl->hostname[GL_MAX_HOSTNAME_LEN] = 0;
-   
-   gl->port = port;
-   
-   if (-1 == login (gl))
-     {
-	free (gl->pseudoname);
-	gl->pseudoname = NULL;
-	
-	return -1;
-     }
-   return 0;
-}
 
 /*}}}*/
 
 
-static int gl_open_predictions (char *group)
+static int gl_open_predictions (GL_Type *gl, char *group) /*{{{*/
 {
-   GL_Type *gl = &GL_Server;
-   
    if (-1 == validate_token (gl))
      return -1;
        
    if (-1 == start_command (gl, "GetPredictions %s %s", gl->token, group))
      return -1;
+
    return 0;
 }
 
-static int terminate_command (GL_Type *gl, GL_Error_Type *tbl)
+/*}}}*/
+
+static int terminate_command (GL_Type *gl, GL_Error_Type *tbl) /*{{{*/
 {
    if (-1 == send_gl_line (gl, ".", 1))
      return -1;
@@ -553,34 +611,31 @@ static int terminate_command (GL_Type *gl, GL_Error_Type *tbl)
    return check_for_error_response (gl, tbl);
 }
 
-static int gl_want_prediction (char *msgid)
+/*}}}*/
+
+static int gl_want_prediction (GL_Type *gl, char *msgid) /*{{{*/
 {
-   GL_Type *gl;
-   
-   gl = &GL_Server;
    return send_gl_line (gl, msgid, 0);
 }
 
-static GL_Error_Type Predictions_Error_Table [] = 
+/*}}}*/
+
+static GL_Error_Type Predictions_Error_Table [] = /*{{{*/
 {
    {":invalidToken",	13,	GL_ERROR_TOKEN},
    {":invalidGroup",	13,	GL_ERROR_NEWSGROUP},
    {NULL, 0, 0}
 };
 
-static int gl_close_predictions (void (*f) (GL_Prediction_Type *))
+/*}}}*/
+
+static int gl_close_predictions (GL_Type *gl, void (*f) (GL_Prediction_Type *)) /*{{{*/
 {  
    char buf [GL_MAX_RESPONSE_LINE_LEN];
    GL_Prediction_Type st;
-   GL_Type *gl;
-
-   gl = &GL_Server;
    
    if (-1 == terminate_command (gl, Predictions_Error_Table))
-     {
-	return -1;
-     }
-   
+     return -1;
    
    while (-1 != get_gl_response (gl, buf, sizeof (buf)))
      {
@@ -628,24 +683,23 @@ static int gl_close_predictions (void (*f) (GL_Prediction_Type *))
    return -1;
 }
 
-static int gl_open_ratings (char *group)
+/*}}}*/
+
+static int gl_open_ratings (GL_Type *gl, char *group) /*{{{*/
 {
-   GL_Type *gl = &GL_Server;
-   
    if (-1 == validate_token (gl))
      return -1;
-   
+
    if (-1 == start_command (gl, "PutRatings %s %s", gl->token, group))
      return -1;
    return 0;
 }
 
-static int gl_put_rating (GL_Rating_Type *rt)
+/*}}}*/
+
+static int gl_put_rating (GL_Type *gl, GL_Rating_Type *rt) /*{{{*/
 {
    char buf [GL_MAX_RESPONSE_LINE_LEN];
-   GL_Type *gl;
-   
-   gl = &GL_Server;
    
    if (rt->rating <= 0) return 0;
    sprintf (buf, "%s :rating=%4.2f :sawHeader=1", rt->msgid, (float) rt->rating);
@@ -655,62 +709,29 @@ static int gl_put_rating (GL_Rating_Type *rt)
    return send_gl_line (gl, buf, 0);
 }
 
-static int gl_close_ratings (void)
+/*}}}*/
+
+static int gl_close_ratings (GL_Type *gl) /*{{{*/
 {
-   GL_Type *gl;
-   
-   gl = &GL_Server;
-   
    if (-1 == terminate_command (gl, Predictions_Error_Table))
      return -1;
-   
+
    close_gl (gl);
    return 0;
 }
 
-static void gl_close_server (void)
-{
-   GL_Type *gl;
-   
-   gl = &GL_Server;
-
-   if (gl->logged_in == 0) 
-     return;
-   
-   (void) logout (gl);
-   
-   if (gl->pseudoname != NULL)
-     free (gl->pseudoname);
-   
-   if (gl->token != NULL)
-     free (gl->token);
-   
-   memset ((char *)gl, 0, sizeof (GL_Type));
-}
-
 /*}}}*/
 
 int Slrn_Use_Group_Lens = 0;
 
-static void do_error (void)
+static void do_error (void) /*{{{*/
 {
    slrn_error ("Failed: %s", gl_get_error ());
 }
 
-char *Slrn_GroupLens_Host;
-int Slrn_GroupLens_Port;
-char *Slrn_GroupLens_Pseudoname;
+/*}}}*/
 
-typedef struct GL_Newsgroup_List_Type
-{
-   char *group;
-   struct GL_Newsgroup_List_Type *next;
-}
-GL_Newsgroup_List_Type;
-
-static GL_Newsgroup_List_Type *Newsgroup_List = NULL;
-
-int slrn_grouplens_add_group (char *group)
+int slrn_grouplens_add_group (char *group) /*{{{*/
 {
    GL_Newsgroup_List_Type *g;
    
@@ -718,7 +739,9 @@ int slrn_grouplens_add_group (char *group)
    g = (GL_Newsgroup_List_Type *) malloc (sizeof (GL_Newsgroup_List_Type));
    if (g == NULL)
      return -1;
-   
+
+   memset ((char *) g, 0, sizeof (GL_Newsgroup_List_Type));
+
    g->group = slrn_safe_strmalloc (group);
    g->next = Newsgroup_List;
    Newsgroup_List = g;
@@ -726,7 +749,102 @@ int slrn_grouplens_add_group (char *group)
    return 0;
 }
 
-static int is_group_valid (char *name)
+/*}}}*/
+
+static void free_gl (GL_Type *gl) /*{{{*/
+{
+   if (gl == NULL)
+     return;
+
+   if (gl->pseudoname != NULL)
+     free (gl->pseudoname);
+   
+   if (gl->hostname != NULL)
+     free (gl->hostname);
+   
+   if (gl->token != NULL)
+     free (gl->token);
+
+   free (gl);
+}
+
+/*}}}*/
+
+static GL_Type *create_gl_for_group (char *group) /*{{{*/
+{
+   GL_Type *gl;
+   char response [GL_MAX_RESPONSE_LINE_LEN];
+   unsigned int value_len;
+   char *value;
+   int port;
+
+   gl = &GL_Nameserver;
+   
+   if (-1 == gl_send_command (gl, NULL, 
+			      "lookupbroker :partitionname=%s\r\n", group))
+     return NULL;
+   
+   if (-1 == get_gl_response (gl, response, sizeof(response)))
+     return NULL;
+   
+   close_gl (gl);
+   
+   /* We expect something like:
+    * lookupbroker :partitionname=comp.unix.questions \
+    *   :host=grouplens.cs.umn.edu :port=42828  :protocol_version=a1
+    */
+   
+   if (-1 == parse_keyword_eqs_value (response, ":port", &value, &value_len))
+     {
+	GL_Error = GL_ERROR_PROPER_RESPONSE;
+	return NULL;
+     }
+   port = atoi (value);
+   
+   if (-1 == parse_keyword_eqs_value (response, ":host", &value, &value_len))
+     {
+	GL_Error = GL_ERROR_PROPER_RESPONSE;
+	return NULL;
+     }
+
+   gl = GL_Server_List;
+   while (gl != NULL)
+     {
+	if ((gl->port == port)
+	    && (0 == strncmp (value, gl->hostname, value_len))
+	    && (gl->hostname [value_len] == 0))
+	  return gl;
+	
+	gl = gl->next;
+     }
+   
+   if (NULL == (gl = (GL_Type *) malloc (sizeof (GL_Type))))
+     return NULL;
+
+   memset ((char *) gl, 0, sizeof (GL_Type));
+   gl->port = port;
+
+   if (NULL == (gl->hostname = make_nstring (value, value_len)))
+     {
+	free_gl (gl);
+	return NULL;
+     }
+
+   if (NULL == (gl->pseudoname = make_string (Slrn_GroupLens_Pseudoname)))
+     {
+	free_gl (gl);
+	return NULL;
+     }
+   
+   gl->next = GL_Server_List;
+   GL_Server_List = gl;
+
+   return gl;
+}
+
+/*}}}*/
+
+static GL_Type *is_group_valid (char *name) /*{{{*/
 {
    GL_Newsgroup_List_Type *g;
    
@@ -734,81 +852,192 @@ static int is_group_valid (char *name)
    while (g != NULL)
      {
 	if (0 == slrn_case_strcmp ((unsigned char *)name, (unsigned char *)g->group))
-	  return 1;
-	
+	  {
+	     if (g->gl == NULL)
+	       g->gl = create_gl_for_group (name);
+	     return g->gl;
+	  }
 	g = g->next;
      }
+   return NULL;
+}
+
+/*}}}*/
+
+static int gl_pseudonym_fun (unsigned int argc, char **argv)
+{
+   switch (argc)
+     {
+      case 4:
+	if (slrn_case_strcmp ((unsigned char *)argv[1], (unsigned char *)"GROUP"))
+	  break;
+	return 0;
+
+      case 3:
+	if (slrn_case_strcmp ((unsigned char *)argv[1], (unsigned char *)"DEFAULT"))
+	  break;
+	/* drop */
+      case 2:
+	slrn_free (Slrn_GroupLens_Pseudoname);
+	Slrn_GroupLens_Pseudoname = slrn_safe_strmalloc (argv[argc - 1]);
+	return 0;
+     }
+   
+   slrn_error ("%s usage: %s [default] PSEUDONYM", argv[0], argv[0]);
+   return -1;
+}
+
+static int gl_glnshost_fun (unsigned int argc, char **argv)
+{
+   if (argc != 2)
+     {
+	slrn_error ("%s: expecting a single argument", argv[0]);
+	return -1;
+     }
+   slrn_free (Slrn_GroupLens_Host);
+   Slrn_GroupLens_Host = slrn_safe_strmalloc (argv[1]);
    return 0;
 }
 
-static int read_grplens_file (void)
+static int gl_glnsport_fun (unsigned int argc, char **argv)
 {
-   char file [256];
-   char *name = ".grplens";
+   if (argc != 2)
+     {
+	slrn_error ("%s: expecting a single argument", argv[0]);
+	return -1;
+     }
+   if (1 != sscanf (argv[1], "%d", &Slrn_GroupLens_Port))
+     Slrn_GroupLens_Port = -1;
+   return 0;
+}
+
+typedef struct 
+{
+#define GL_MAX_CONFIG_FILE_ARGC 10
+   char *name;
+   int (*fun) (unsigned int, char **);
+}
+GL_Config_File_Type;
+
+static int parse_to_argc_argv (char *line, unsigned int *argc_p,
+			       char **argv, unsigned int max_argc)
+{
+   unsigned int argc;
+   
+   argc = 0;
+   while (argc <= max_argc)
+     {
+	line = slrn_skip_whitespace (line);
+	if (*line == 0)
+	  {
+	     argv[argc] = NULL;
+	     *argc_p = argc;
+	     return 0;
+	  }
+
+	/* Later I will modify this to handle quotes. */
+	argv[argc++] = line;
+	while (*line 
+	       && (*line != ' ') 
+	       && (*line != '\t')
+	       && (*line != '\n'))
+	  line++;
+	if (*line) *line++ = 0;
+     }
+   
+   slrn_error ("Too many arguments");
+   return -1;
+}
+
+   
+
+static GL_Config_File_Type GL_Config [] = 
+{
+   {"PSEUDONYM", gl_pseudonym_fun},
+   {"GLNSHOST", gl_glnshost_fun},
+   {"GLNSPORT", gl_glnsport_fun},
+   {"BBBHOST", gl_glnshost_fun},
+   {"BBBPORT", gl_glnsport_fun},
+   {"DISPLAYTYPE", NULL},
+   {NULL, NULL}
+};
+
+
+static int read_grplens_file (void) /*{{{*/
+{
+   char file [SLRN_MAX_PATH_LEN];
    char line [1024];
    FILE *fp;
-   
-   fp = slrn_open_home_file (name, "r", file, 0);
-   if (fp == NULL) return -1;
+   char *argv [GL_MAX_CONFIG_FILE_ARGC + 1];
+   unsigned int argc;
+   unsigned int linenum;
+   GL_Config_File_Type *cfg;
+
+   fp = slrn_open_home_file (".grouplens", "r", file, 0);
+   if (fp == NULL) 
+     {
+	fp = slrn_open_home_file (".grplens", "r", file, 0);
+	if (fp == NULL)
+	  return -1;
+     }
 
    slrn_message ("Reading %s", file);
-   
+   linenum = 0;
+
    while (NULL != fgets (line, sizeof (line), fp))
      {
 	char *b;
 	
+	linenum++;
 	b = slrn_skip_whitespace (line);
 	if ((*b == 0) || (*b == '#') || (*b == '%'))
 	  continue;
-	
-	(void) slrn_trim_string (b);
-	
-	if (!slrn_case_strncmp ((unsigned char *)b, (unsigned char *) "BBBHOST", 7))
+
+	if (-1 == parse_to_argc_argv (b, &argc, argv, GL_MAX_CONFIG_FILE_ARGC))
 	  {
-	     b = slrn_skip_whitespace (b + 7);
-	     if (*b == 0) continue;
-	     slrn_free (Slrn_GroupLens_Host);
-	     Slrn_GroupLens_Host = slrn_safe_strmalloc (b);
-	     continue;
+	     slrn_error ("Error processing line %u of %s", linenum, file);
+	     return -1;
 	  }
 	
-	if (!slrn_case_strncmp ((unsigned char *)b, (unsigned char *)"PSEUDONYM", 9))
-	  {
-	     b = slrn_skip_whitespace (b + 9);
-	     if (*b == 0) continue;
-	     slrn_free (Slrn_GroupLens_Pseudoname);
-	     Slrn_GroupLens_Pseudoname = slrn_safe_strmalloc (b);
-	     continue;
-	  }
-	
-	if (!slrn_case_strncmp ((unsigned char *) b, (unsigned char *)"BBBPORT", 7))
-	  {
-	     b = slrn_skip_whitespace (b + 7);
-	     if (*b == 0) continue;
-	     if (1 != sscanf (b, "%d", &Slrn_GroupLens_Port))
-	       Slrn_GroupLens_Port = -1;
-	     
-	     continue;
-	  }
-	
-	if (!slrn_case_strncmp ((unsigned char *)b, (unsigned char *)"DISPLAYTYPE", 11))
+	if (argc == 0)
 	  continue;
 	
-	/* Anything else is a newsgroup */
+	cfg = GL_Config;
+	while (cfg->name != NULL)
+	  {
+	     if (!slrn_case_strcmp ((unsigned char *)cfg->name, 
+				     (unsigned char *)argv[0]))
+	       break;
+	     cfg++;
+	  }
 	
-	(void) slrn_grouplens_add_group (b);
+	if (cfg->name == NULL)
+	  {
+	     /* Anything else is a newsgroup */
+	     (void) slrn_grouplens_add_group (argv[0]);
+	     continue;
+	  }
+	
+	if ((cfg->fun != NULL) 
+	    && (-1 == (*cfg->fun) (argc, argv)))
+	  {	
+	     slrn_error ("Error processing line %u of %s", linenum, file);
+	     return -1;
+	  }
      }
    
    fclose (fp);
    return 0;
 }
 
-int slrn_init_grouplens (void)
+/*}}}*/
+
+int slrn_init_grouplens (void) /*{{{*/
 {
    Slrn_Use_Group_Lens = 0;
 
    (void) read_grplens_file ();
-   
+
    if ((Slrn_GroupLens_Host == NULL)
        || (Slrn_GroupLens_Port <= 0)
        || (Slrn_GroupLens_Pseudoname == NULL))
@@ -816,32 +1045,67 @@ int slrn_init_grouplens (void)
 	return -1;
      }
    
-   slrn_message ("Connecting to GroupLens host %s, port %d ...", Slrn_GroupLens_Host, Slrn_GroupLens_Port);
-   
-   if (-1 == gl_open_server (Slrn_GroupLens_Host, Slrn_GroupLens_Port, Slrn_GroupLens_Pseudoname))
-     {
-	slrn_error ("GroupLens login failure: %s", gl_get_error ());
-	return -1;
-     }
-   
+   slrn_message ("Checking to see if server %s:%d is alive...", 
+		 Slrn_GroupLens_Host, Slrn_GroupLens_Port);
+   if (-1 == connect_to_gl_host (&GL_Nameserver, NULL))
+     return -1;
+   close_gl (&GL_Nameserver);
+
    Slrn_Use_Group_Lens = 1;
    slrn_message ("GroupLens support initialized.");
    return 0;
 }
 
-void slrn_close_grouplens (void)
+/*}}}*/
+
+static void gl_close_servers (void) /*{{{*/
+{
+   GL_Newsgroup_List_Type *g, *gnext;
+   GL_Type *gl, *glnext;
+
+   g = Newsgroup_List;
+   while (g != NULL)
+     {
+	gnext = g->next;
+	if (g->group != NULL)
+	  free (g->group);
+	
+	free (g);
+	g = gnext;
+     }
+   Newsgroup_List = NULL;
+   
+   gl = GL_Server_List;
+   while (gl != NULL)
+     {
+	glnext = gl->next;
+	(void) logout (gl);
+	free_gl (gl);
+	gl = glnext;
+     }
+   GL_Server_List = NULL;
+}
+
+/*}}}*/
+
+void slrn_close_grouplens (void) /*{{{*/
 {
    if (Slrn_Use_Group_Lens == 0) return;
-   slrn_message ("Logging out of GroupLens server ...");
+   slrn_message ("Logging out of GroupLens servers ...");
+   
    Slrn_Message_Present = 0;
    Slrn_Use_Group_Lens = 0;
-   gl_close_server ();
+   
+   gl_close_servers ();
+   slrn_message ("");
 }
 
+/*}}}*/
+
 static int Did_Rating = 0;
 static int Prediction_Count;
 
-static void prediction_callback (GL_Prediction_Type *s)
+static void prediction_callback (GL_Prediction_Type *s) /*{{{*/
 {
    Slrn_Header_Type *h;
    int i;
@@ -859,23 +1123,27 @@ static void prediction_callback (GL_Prediction_Type *s)
    Prediction_Count++;
 }
 
-int slrn_get_grouplens_scores (void)
+/*}}}*/
+
+int slrn_get_grouplens_scores (void) /*{{{*/
 {
    Slrn_Header_Type *h;
    int ret;
-   
+   GL_Type *gl;
+
    if ((Slrn_Use_Group_Lens == 0) 
        || (Slrn_First_Header == NULL))
      return -1;
 
-   if (0 == is_group_valid (Slrn_Current_Group_Name))
+   if (NULL == (gl = is_group_valid (Slrn_Current_Group_Name)))
      return -1;
    
    Prediction_Count = 0;
    
-   slrn_message_now ("Getting GroupLens predictions ...");
+   slrn_message_now ("Getting GroupLens predictions from %s:%d...", 
+		     gl->hostname, gl->port);
  
-   if (-1 == gl_open_predictions (Slrn_Current_Group_Name))
+   if (-1 == gl_open_predictions (gl, Slrn_Current_Group_Name))
      {
 	do_error ();
 	return -1;
@@ -886,7 +1154,7 @@ int slrn_get_grouplens_scores (void)
    ret = 0;
    while (h != NULL)
      {
-	if (ret == 0) ret = gl_want_prediction (h->msgid);
+	if (ret == 0) ret = gl_want_prediction (gl, h->msgid);
 	h = h->real_next;
      }
    
@@ -896,7 +1164,7 @@ int slrn_get_grouplens_scores (void)
 	return -1;
      }
    
-   if (-1 == gl_close_predictions (prediction_callback))
+   if (-1 == gl_close_predictions (gl, prediction_callback))
      {
 	do_error ();
 	return -1;
@@ -905,11 +1173,13 @@ int slrn_get_grouplens_scores (void)
    return Prediction_Count;
 }
 
-     
-int slrn_put_grouplens_scores (void)
+/*}}}*/
+
+int slrn_put_grouplens_scores (void) /*{{{*/
 {  
    Slrn_Header_Type *h;
-   
+   GL_Type *gl;
+
    if (Slrn_Use_Group_Lens == 0) return 0;
    if (Did_Rating == 0) return 0;
    Did_Rating = 0;
@@ -917,9 +1187,13 @@ int slrn_put_grouplens_scores (void)
    h = Slrn_First_Header;
    if (h == NULL) return 0;
    
-   slrn_message_now ("Sending GroupLens ratings ...");
-   
-   if (-1 == gl_open_ratings (Slrn_Current_Group_Name))
+   if (NULL == (gl = is_group_valid (Slrn_Current_Group_Name)))
+     return -1;
+
+   slrn_message_now ("Sending GroupLens ratings to %s:%d...", 
+		     gl->hostname, gl->port);
+
+   if (-1 == gl_open_ratings (gl, Slrn_Current_Group_Name))
      {
 	do_error ();
 	return -1;
@@ -943,7 +1217,7 @@ int slrn_put_grouplens_scores (void)
 		  r.rating = h->gl_rating;
 	       }
 	     
-	     if (-1 == gl_put_rating (&r))
+	     if (-1 == gl_put_rating (gl, &r))
 	       {
 		  do_error ();
 		  return -1;
@@ -952,7 +1226,7 @@ int slrn_put_grouplens_scores (void)
 	h = h->real_next;
      }
    
-   if (-1 == gl_close_ratings ())
+   if (-1 == gl_close_ratings (gl))
      {
 	do_error ();
 	return -1;
@@ -961,13 +1235,16 @@ int slrn_put_grouplens_scores (void)
    return 0;
 }
 
-void slrn_group_lens_rate_article (Slrn_Header_Type *h, int score, int saw_article)
+/*}}}*/
+
+void slrn_group_lens_rate_article (Slrn_Header_Type *h, int score, int saw_article) /*{{{*/
 {
    if (saw_article) score = score * 10;
    h->gl_rating = score;
    Did_Rating++;
 }
 
+/*}}}*/
 
 		
 #endif				       /* SLRN_HAS_GROUPLENS */
