@@ -529,7 +529,7 @@ static int generic_search_article (char *str, int is_regexp) /*{{{*/
    if (l == NULL)
      return 0;
    
-   SLang_push_string (l->buf);
+   (void) SLang_push_string (l->buf);
    return 1;
 }
 
@@ -909,7 +909,7 @@ static char *current_group_name (void)
    if (Slrn_Group_Current_Group == NULL)
      return "";
    
-   return Slrn_Group_Current_Group->name;
+   return Slrn_Group_Current_Group->group_name;
 }
 
 static int get_group_unread_count (void)
