@@ -80,4 +80,28 @@
 # define SLREALLOC realloc
 #endif
 
+#include <ctype.h>
+
+#if !defined(HAVE_ISALPHA) && !defined(isalpha)
+# define isalpha(x) \
+  ((((x) <= 'Z') && ((x) >= 'A')) \
+   || (((x) <= 'z') && ((x) >= 'a')))
+#endif
+
+#if !defined(HAVE_ISSPACE) && !defined(isspace)
+# define isspace(x) (((x)==' ')||((x)=='\t')||((x)=='\n')||((x)=='\r'))
+#endif
+
+#if !defined(HAVE_ISDIGIT) && !defined(isdigit)
+# define isdigit(x) (((x) <= '9') && ((x) >= '0'))
+#endif
+
+#if !defined(HAVE_ISALNUM) && !defined(isalnum)
+# define isalnum(x) (isalpha(x)||isdigit(x))
+#endif
+
+#if !defined(HAVE_ISPUNCT) && !defined(ispunct)
+# define ispunct(x)  (((x)=='.')||((x)==',')||((x)==';')||((x)=='!')||((x)=='?'))
+#endif
+
 #endif				       /* _JD_MACROS_H_ */
