@@ -478,13 +478,14 @@ static void decode_mime_base64 (void)
 static int merge_if_soft_linebreak (Slrn_Article_Line_Type *curr_line)
 {
    Slrn_Article_Line_Type *next_line;
+   char *b;
 
    while ((next_line = curr_line->next) != NULL)
      {
-	char *b = curr_line->buf;
 	unsigned int len;
 
-	len = strlen (b);
+	b = curr_line->buf;
+	len = (unsigned int) (slrn_bskip_whitespace (b) - b);
 	if (len == 0) return 0;
    
 	len--;
@@ -508,6 +509,16 @@ static int merge_if_soft_linebreak (Slrn_Article_Line_Type *curr_line)
 	SLFREE (next_line->buf);
 	SLFREE (next_line);
      }
+
+   /* In case the last line ends with a soft linebreak: */
+   b = slrn_bskip_whitespace (curr_line->buf);
+   if (b != curr_line->buf)
+     {
+	b--;
+	if (*b == '=')
+	  *b = 0;
+     }
+   
    return 0;
 }
 
@@ -531,7 +542,7 @@ static void decode_mime_quoted_printable (Slrn_Article_Type *a)
 	unsigned int len;
 	
 	b = line->buf;
-	len = strlen (b);
+	len = (unsigned int) (slrn_bskip_whitespace (b) - b);
 	if (len && (b[len - 1] == '=') 
 	    && (line->next != NULL))
 	  {
