@@ -23,6 +23,7 @@
 #include <slang.h>
 
 #include "ttymsg.h"
+#include "vfile.h"
 
 extern void slrn_make_home_filename (char *, char *);
 extern int slrn_make_home_dirname (char *, char *);
@@ -34,6 +35,7 @@ extern int slrn_get_yesno_cancel (char *str, ...);
 extern void slrn_clear_message (void);
 extern void slrn_print_percent (int, int, SLscroll_Window_Type *);
 extern FILE *slrn_open_home_file (char *, char *, char *, int);
+extern VFILE *slrn_open_home_vfile (char *, char *);
 extern void slrn_suspend_cmd (void);
 extern int slrn_read_integer (char *, int *, int *);
 extern int slrn_read_input (char *, char *, char *, int, int);
