@@ -148,6 +148,10 @@ void slrn_add_signature (FILE *fp)
    char file[SLRN_MAX_PATH_LEN];
    char buf [256];
    
+   if ((Slrn_User_Info.signature == NULL)
+       || (Slrn_User_Info.signature[0] == 0))
+     return;
+
    if ((sfp = slrn_open_home_file (Slrn_User_Info.signature, "r", file, 0)) != NULL)
      {
 	/* Apparantly some RFC suggests the -- \n. */
@@ -339,7 +343,7 @@ static int check_file_for_posting (char *file)
    int newsgroups_found, subject_found, only_warning;
    char ch;
    char *colon;
-   
+
    only_warning = newsgroups_found = subject_found = 0;
    err = NULL;
    fp = fopen (file, "r");
@@ -433,7 +437,9 @@ static int check_file_for_posting (char *file)
      {
 	char *qs = Slrn_Quote_String;
 	unsigned int qlen;
-	
+#if SLRN_HAS_VERBATIM_MARKS
+	int is_verbatim = 0;
+#endif
 	if (qs == NULL) qs = ">";
 	qlen = strlen (qs);
 	
@@ -444,7 +450,25 @@ static int check_file_for_posting (char *file)
 	     num++;
 	     if (!strncmp (line, qs, qlen))
 	       continue;
+#if SLRN_HAS_VERBATIM_MARKS
+	     if (0 == strncmp (line, "#v", 2))
+	       {
+		  if (is_verbatim && (line[2] == '-'))
+		    {
+		       is_verbatim = 0;
+		       continue;
+		    }
+		  
+		  if ((is_verbatim == 0) && (line[2] == '+'))
+		    {
+		       is_verbatim = 1;
+		       continue;
+		    }
+	       }
 	     
+	     if (is_verbatim)
+	       continue;
+#endif
 	     colon = slrn_skip_whitespace (line);
 	     if (*colon == 0) continue;
 	     
