@@ -946,6 +946,12 @@ static void read_score_file (void)
    
    slrn_make_home_filename (Slrn_Score_File, file);
    
+   if (1 != slrn_file_exists (file))
+     {
+	slrn_message_now ("*** Warning: Score file %s does not exist", file);
+	return;
+     }
+
    if (-1 == slrn_read_score_file (file))
      {
 	slrn_exit_error ("Error processing score file %s.", file);
@@ -1128,6 +1134,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 
    if (Slrn_Server_Id == 0) Slrn_Server_Id = Slrn_Default_Server_Obj;
    if (Slrn_Post_Id == 0) Slrn_Post_Id = Slrn_Default_Post_Obj;
+   if (Slrn_Check_New_Groups == 0) no_new_groups = 1;
 
 #if SLRN_HAS_CHARACTER_MAP
    if (-1 == slrn_set_charset (Slrn_Charset))
