@@ -43,12 +43,12 @@
 #define SLRN_HAS_EMPHASIZED_TEXT	1
 
 /* If non-zero, slrn will treat specially marked lines of an article 
- * as verbatum text.  It uses #v+ and #v- as markers of such text, e.g.,
+ * as verbatim text.  It uses #v+ and #v- as markers of such text, e.g.,
  * #v+
- *    This line denotes verbatum text.
+ *    This line denotes verbatim text.
  * #v-
  */
-#define SLRN_HAS_VERBATUM_MARKS	1
+#define SLRN_HAS_VERBATIM_MARKS	1
 
 /* See README.GroupLens.  This only works if SLRN_USE_SLTCP is non-zero. 
  * Also make sure NNTP support is enabled.
