@@ -724,15 +724,11 @@ int sltcp_fgets (SLTCP_Type *tcp, char *buf, unsigned int len) /*{{{*/
 
 /*}}}*/
 
-#if defined(USE_WINSOCK_SLTCP)
-static WSADATA wsaData;
-static int Winsock_Started = 0;
-#endif
-
 /* Before any of the above routines in this file may be used, sltcp_open_sltcp
  * must be called.
  */
 #if defined(USE_WINSOCK_SLTCP)
+static int Winsock_Started = 0;
 static void w32_close_sltcp (void)
 {
    (void) sltcp_close_sltcp ();
@@ -742,7 +738,8 @@ static void w32_close_sltcp (void)
 int sltcp_open_sltcp (void)
 {
 #if defined(USE_WINSOCK_SLTCP)
-   WORD wVerReq = MAKEWORD(1, 1);
+   WSADATA wsaData;
+   WORD wVerReq = MAKEWORD(1, 1);      /* version 1.1, for 2.0, use (2,0) */
 
    if (Winsock_Started)
      return 0;
@@ -761,7 +758,7 @@ int sltcp_close_sltcp (void)
 #if defined(USE_WINSOCK_SLTCP)
    if (Winsock_Started == 0)
      return 0;
-
+   WSACancelBlockingCall();	       /* cancel any blocking calls */
    WSACleanup ();
    Winsock_Started = 0;
 #endif
@@ -769,3 +766,35 @@ int sltcp_close_sltcp (void)
    return 0;
 }
 
+#if 0 && defined(USE_WINSOCK_SLTCP)
+/* Some winsock notes */
+
+/* To interrupt a read/write on a socket, it may be useful to set a handler
+ * control C, e.g., define something like
+ * 
+ *    SetConsoleCtrlHandler (kbd_int_handler, TRUE);
+ *
+ * where:
+ */
+static BOOL WINAPI kbd_int_handler (DWORD type)
+{
+   BOOL ret = TRUE;
+
+   switch (type)
+     {
+      case CTRL_C_EVENT:                     /* ctrl+c hit */
+      case CTRL_BREAK_EVENT:                 /* ctrl+break hit */
+	ret = FALSE;
+	break;
+      case CTRL_CLOSE_EVENT:                 /* window closing */
+      case CTRL_LOGOFF_EVENT:                /* user logoff */
+      case CTRL_SHUTDOWN_EVENT:              /*system shutdown */
+      default:
+	ret = TRUE;
+	break;
+     }
+   (void) w32_close_sltcp ();
+   return ret;
+}
+
+#endif
