@@ -331,11 +331,13 @@ Set_Int_Type;
 
 static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 {
+     {"hide_verbatim_marks",	&Slrn_Verbatim_Marks_Hidden},
      {"hide_signature",	&Slrn_Signature_Hidden},
      {"hide_pgpsignature",	&Slrn_Pgp_Signature_Hidden},
      {"emphasized_text_mask",	&Slrn_Emphasized_Text_Mask},
      {"emphasized_text_mode",	&Slrn_Emphasized_Text_Mode},
-     {"process_verbatum_marks", &Slrn_Process_Verbatum_Marks},
+     {"process_verbatim_marks", &Slrn_Process_Verbatim_Marks},
+     {"process_verbatum_marks", &Slrn_Process_Verbatim_Marks},
 
      {"use_flow_control",	&Slrn_Use_Flow_Control},
      {"abort_unmodified_edits", &Slrn_Abort_Unmodified},
@@ -369,6 +371,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"no_backups", &Slrn_No_Backups},
      {"beep", &SLtt_Ignore_Beep},
      {"unsubscribe_new_groups", &Slrn_Unsubscribe_New_Groups},
+     {"check_new_groups", &Slrn_Check_New_Groups},
      {"show_thread_subject", &Slrn_Show_Thread_Subject},
      {"mouse", &Slrn_Use_Mouse},
      {"query_next_group", &Slrn_Query_Next_Group},
@@ -706,23 +709,24 @@ Color_Handle_Type;
 
 static Color_Handle_Type Color_Handles[] = /*{{{*/
 {
-     {"article",	ARTICLE_COLOR,		"blue",		DEF_BG, 0},
+     {"article",	ARTICLE_COLOR,		DEF_FG,		DEF_BG, 0},
      {"author",		AUTHOR_COLOR,		"magenta",	DEF_BG, 0},
      {"boldtext", 	BOLD_COLOR,		"yellow", 	DEF_BG, SLTT_BOLD_MASK},
      {"box", 		BOX_COLOR, 		DEF_FG, 	"blue", 0},
      {"cursor",		CURSOR_COLOR,		"brightgreen",	DEF_BG, SLTT_REV_MASK},
+     {"date",		DATE_COLOR,		"magenta",	DEF_BG, 0},
      {"description",	GROUP_DESCR_COLOR,	"magenta",	DEF_BG, 0},
      {"error",		ERROR_COLOR,		"red",		DEF_BG, SLTT_BLINK_MASK},
      {"frame", 		FRAME_COLOR,		"yellow", 	"blue", SLTT_REV_MASK},
-     {"group",		GROUP_COLOR,		"blue",		DEF_BG, 0},
-     {"grouplens_display",GROUPLENS_DISPLAY_COLOR,"blue",	DEF_BG, 0},
+     {"group",		GROUP_COLOR,		DEF_FG,		DEF_BG, 0},
+     {"grouplens_display",GROUPLENS_DISPLAY_COLOR,DEF_FG,	DEF_BG, 0},
      {"header_name",	SLRN_HEADER_KEYWORD_COLOR,"green", 	DEF_BG, SLTT_BOLD_MASK},
      {"header_number",	HEADER_NUMBER_COLOR,	"green",	DEF_BG, 0},
      {"headers",	HEADER_COLOR,		"brightcyan",	DEF_BG, 0},
      {"high_score",	HIGH_SCORE_COLOR,	"red",		DEF_BG, SLTT_BOLD_MASK},
      {"italicstext", 	ITALICS_COLOR,		"magenta", 	DEF_BG, 0},
      {"menu",		MENU_COLOR,		"yellow",	"blue", SLTT_REV_MASK},
-     {"menu_press",	MENU_PRESS_COLOR,	"blue",		"yellow", 0},
+     {"menu_press",	MENU_PRESS_COLOR,	DEF_FG,		"yellow", 0},
      {"normal",		0,			DEF_FG,		DEF_BG, 0},
      {"pgpsignature",	PGP_SIGNATURE_COLOR,	"red",		DEF_BG, 0},
      {"quotes",		QUOTE_COLOR,		"red",		DEF_BG, 0},
@@ -731,11 +735,12 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"signature",	SIGNATURE_COLOR,	"red",		DEF_BG, 0},
      {"status",		STATUS_COLOR,		"yellow",	"blue", SLTT_REV_MASK},
      {"subject",	SUBJECT_COLOR,		DEF_FG,		DEF_BG, 0},
-     {"thread_number",	THREAD_NUM_COLOR,	"blue",		DEF_BG, SLTT_BOLD_MASK},
+     {"thread_number",	THREAD_NUM_COLOR,	DEF_FG,		DEF_BG, SLTT_BOLD_MASK},
      {"tilde",		SLRN_TILDE_COLOR,	"green",	DEF_BG, SLTT_BOLD_MASK},
      {"tree",		TREE_COLOR,		"red",		DEF_BG, 0},
-     {"underlinetext", 	UNDERLINE_COLOR,	"magenta", 	DEF_BG, SLTT_BOLD_MASK},
-     {"verbatum", 	VERBATUM_COLOR,		"green", 	DEF_BG, 0},
+     {"underlinetext", 	UNDERLINE_COLOR,	"magenta", 	DEF_BG, SLTT_ULINE_MASK},
+     {"verbatum", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0},
+     {"verbatim", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0},
 
      {NULL, -1, NULL, NULL, 0}
 };
@@ -1173,6 +1178,7 @@ void slrn_startup_initialize (void) /*{{{*/
    (void) slrn_set_header_format (1, "%F%G%-5l:[%12r]%t%s");
    (void) slrn_set_header_format (2, "%F%-5l:%t%s");
    (void) slrn_set_header_format (3, "%F%-5S%-5l:%t%50s %r");
+   (void) slrn_set_header_format (4, "%F%-5S [%10r]:%t%49s %-19g[%17d]");
    
 }
 
