@@ -43,47 +43,45 @@
 #include "util.h"
 #include "ttymsg.h"
 
-
 /* This function allows NULL as a parameter. This fact _is_ exploited */
 char *slrn_skip_whitespace (char *b) /*{{{*/
 {
-   register char ch;
-   
    if (b == NULL) return NULL;
-
-   while (((ch = *b) == ' ') || (ch == '\t') || (ch == '\n'))
+   
+   while (isspace (*b))
      b++;
+
    return b;
 }
 
 /*}}}*/
 
-/* returns a pointer to the end of the string */
-char *slrn_trim_string (char *smin) /*{{{*/
+char *slrn_bskip_whitespace (char *smin)
 {
-   register char *s, ch;
-   
+   char *s;
+
    if (smin == NULL) return NULL;
    s = smin + strlen (smin);
-   
+
    while (s > smin)
      {
 	s--;
-	ch = *s;
-	if ((ch == ' ')
-	    || (ch == '\n')
-	    || (ch == '\t'))
-	  {
-	     *s = 0;
-	     continue;
-	  }
-	
-	s++;
-	break;
+	if (0 == isspace(*s))
+	  return s + 1;
      }
    return s;
 }
 
+   
+/* returns a pointer to the end of the string */
+char *slrn_trim_string (char *s) /*{{{*/
+{
+   s = slrn_bskip_whitespace (s);
+   if (s != NULL)
+     *s = 0;
+
+   return s;
+}
 /*}}}*/
 
 char *slrn_strchr (char *s, char ch) /*{{{*/
@@ -143,17 +141,14 @@ char *slrn_simple_strtok (char *s, char *chp) /*{{{*/
 /*}}}*/
 
 
-/* Note!!!  These routines assume a flat address space !! */
-
 int slrn_case_strncmp (unsigned char *a, register unsigned char *b, register unsigned int n) /*{{{*/
 {
    register unsigned char cha, chb, *bmax;
-   register long diff = a - b;
    
    bmax = b + n;
    while (b < bmax)
      {
-	cha = UPPER_CASE(b[diff]);
+	cha = UPPER_CASE(*a);
 	chb = UPPER_CASE(*b);
 	if (cha != chb)
 	  {
@@ -161,6 +156,7 @@ int slrn_case_strncmp (unsigned char *a, register unsigned char *b, register uns
 	  }
 	else if (chb == 0) return 0;
 	b++;
+	a++;
      }
    return 0;
 }
@@ -170,11 +166,10 @@ int slrn_case_strncmp (unsigned char *a, register unsigned char *b, register uns
 int slrn_case_strcmp (unsigned char *a, register unsigned char *b) /*{{{*/
 {
    register unsigned char cha, chb;
-   register long diff = a - b;
    
    while (1)
      {
-	cha = UPPER_CASE(b[diff]);
+	cha = UPPER_CASE(*a);
 	chb = UPPER_CASE(*b);
 	if (cha != chb)
 	  {
@@ -182,6 +177,7 @@ int slrn_case_strcmp (unsigned char *a, register unsigned char *b) /*{{{*/
 	  }
 	else if (chb == 0) break;
 	b++;
+	a++;
      }
    return 0;
 }
