@@ -810,6 +810,10 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 	     if (*buf == 'M')
 	       {
 		  if ((3 * buflen) != (len * 4)) continue;
+#if 0
+		  if (0 == slrn_case_strncmp (buf, "Message-Id:", 11))
+		    continue;
+#endif
 	       }
 	     else
 	       {
@@ -825,11 +829,10 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 #endif
 		       continue;
 		    }
-		  /* This may still be a bogus line, e.g., a header. */
-		  if (NULL != slrn_strchr (buf, ' '))
-		    continue;
 	       }
-	     
+	     /* This may still be a bogus line, e.g., a header. */
+	     if (NULL != slrn_strchr (buf, ' '))
+	       continue;
 	     
 	     b = (unsigned char *) buf + 1;
 	     bmax = b + buflen;
